#include "StdAfx.h"
#include "WASListBanner.h"
#include "ConvertUTF.h"
#include "MessengerActor.h"
#include "UtilFunction.h"
#include "GameTalkAPIGlobal.h"


CWASListBanner::CWASListBanner(CWASActor *pActor) : CWASCommand(pActor)
{
}

CWASListBanner::~CWASListBanner(void)
{
}


//////////////////////////////////////
void CWASListBanner::ExecuteAsync()
{
	m_listBannerData.clear();		//  Ʈ	

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	USES_UTF8_CONVERSION;
	
	WSTRING strURL = m_pActor->GetWASServerAddress();
	strURL += _T("/ne/notice/ListBannerImage.php");

	m_httpFacade.StopRequest();
	m_httpFacade.session().SetURL( W2U8(strURL.c_str()), pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().SetMethod(HTTP_METHOD_POST);
	m_httpFacade.session().AddBodyVariable("userid", W2U8(pMessengerActor->GetUserID()));	
	m_httpFacade.session().AddBodyVariable("ticket", pMessengerActor->GetMessengerTicket());	

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}


bool CWASListBanner::ParseAndEventFire(nfc::CIkXMLDom * pDom)
{
	if(pDom == NULL)
		return false;

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return false;
	}	

	USES_UTF8_CONVERSION;

	do
	{
		nfc::IIkXMLNode * pRootNode = pDom->GetdocumentElement()->selectSingleNode("nodes");
		nfc::IIkXMLNodeList *pNodeList;
		nfc::IIkXMLNode * pListNode;
		nfc::IIkXMLAttribute * pAttr;		

		pNodeList = pRootNode->selectNodes("node");

		// Tree Re-Construction
		
		for(int j = 0; j < pNodeList->Getlength(); j++)
		{
			NE_BANNER_DATA bannerData;
			memset(&bannerData, 0x00, sizeof(NE_BANNER_DATA));
			pListNode = pNodeList->Getitem(j);

			pAttr = pListNode->getAttribute("seq");
			if(NULL == pAttr) continue;
			bannerData.nSeq = atoi(pAttr->Gettext().c_str());

			pAttr = pListNode->getAttribute("title");
			if(NULL == pAttr) continue;
			bannerData.strTitle = U82W(pAttr->Gettext().c_str());

			pAttr = pListNode->getAttribute("startdate");
			if(NULL == pAttr) continue;
			bannerData.strStartDate = U82W(pAttr->Gettext().c_str());

			pAttr = pListNode->getAttribute("enddate");
			if(NULL == pAttr) continue;
			bannerData.strEndDate = U82W(pAttr->Gettext().c_str());

			pAttr = pListNode->getAttribute("banner_img_url");
			if(NULL == pAttr) continue;
			bannerData.strBannerImageURL = U82W(pAttr->Gettext().c_str());

			pAttr = pListNode->getAttribute("banner_content_url");
			if(NULL == pAttr) continue;
			bannerData.strBannerContentURL = U82W(pAttr->Gettext().c_str());

			m_listBannerData.push_back(bannerData);				
		}

	} while(FALSE);

	return true;
}


// override in drived class
void CWASListBanner::OnWasResponse(trid_t trid,
								  bool bSuccess,
								  int /*nWinINetErrorCode*/, 
								  int /*nHttpResponseCode*/, 
                                  LPCSTR /*szWasResCodeInHeader*/, 
                                  LPCSTR szWasResCodeInXml,
                                  nfc::CIkXMLDom * pDom)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(pDom) == false)
			m_eError = wasError_Protocol;
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
		if((*it)->OnGTWASActor_ResponseBannerList(m_pActor, GetID(), m_eError, m_listBannerData) == false)
			break;

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASListBanner::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();	

	for (; it != observers.end(); ++it)
		if((*it)->OnGTWASActor_ResponseBannerList(m_pActor, GetID(), m_eError, m_listBannerData) == false)
			break;

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}
